
module Steam
  module Id
    # Represents a Steam ID in the Steam2 format.
    class Steam2Id < Base
      # Parse a Steam2Id from a given input string
      #
      # @param id [String] the input string
      # @return [Steam2Id, nil]
      def self.parse(id)
        parser = Steam2IdParser.new(id)
        parser.parse

        new(parser.universe, parser.account_id, parser.auth_server)
      end

      # Instantiate a Steam2Id object
      def initialize(universe, account_id, auth_server)
        super()
        self.instance = 1
        self.universe = universe
        self.account_type = EAccountType::INDIVIDUAL
        self.account_id = (account_id << 1) | auth_server
      end

      # Sets the correct universe for a Steam2Id
      #
      # @param universe [Integer] the universe
      def universe=(universe)
        struct.universe = universe.zero? ? EUniverse::PUBLIC : universe
      end
    end
  end
end
