module Steam
  module Id
    # Parse a steam 2 id string
    class Steam2IdParser
      # A Regex representing a valid Steam 2 string
      # rubocop:disable LineLength
      STEAM2_REGEX = /STEAM_(?<universe>[0-4]):(?<authserver>[0-1]):(?<accountid>\d+)/

      attr_reader :universe, :account_id, :auth_server

      def initialize(id)
        @id = id.to_s
        @universe = EUniverse::INVALID
        @account_id = 0
        @auth_server = 0
      end

      # Parse a Steam2Id from a given input string
      def parse
        return unless matches
        @universe   = Integer(matches['universe'])
        @account_id = Integer(matches['accountid'])
        @auth_server = Integer(matches['authserver'])
      end

      private

      # @api private
      def matches
        @matches ||= @id.match(STEAM2_REGEX)
      end
    end
  end
end
