module Steam
  module Id
    # Parse a steam 3 id string
    class Steam3IdParser
      # A Regex representing a valid Steam 3 string
      # rubocop:disable LineLength
      STEAM3_REGEX = /\[(?<type>[AGMPCgcLTIUai]):(?<universe>[0-4]):(?<account>\d+)(:(?<instance>\d+))?\]/

      attr_reader :account_id
      attr_reader :universe
      attr_reader :type

      def initialize(id)
        @id = id.to_s
        @type = nil
        @account_id = 0
        @universe = EUniverse::INVALID
        @type = 'U'
      end

      # Create a Steam3 id by parsing an input string
      def parse
        return unless matches

        @account_id = Integer(matches['account'])
        @universe = Integer(matches['universe'])
        @type = matches['type']
      end

      # The instance of the account. Currently only instance 1 is supported
      def instance
        1
      end

      # The type of account as an integer
      def type
        case @type
        when 'U'
          EAccountType::INDIVIDUAL
        else
          EAccountType::INVALID
        end
      end

      private

      # @api private
      def matches
        @matches ||= @id.match(STEAM3_REGEX)
      end
    end
  end
end
