module Steam
  module Id
    # The internal datastructure of a steamid. A 64 bit integer
    class Struct
      attr_reader :data

      # The bit mask applied to get the account id
      ACCOUNT_ID_MASK   = 0xFFFFFFFF

      # The bit mask applied to get the instance mask
      INSTANCE_MASK     = 0xFFFFF

      # The bit mask applied to get the universe mask
      UNIVERSE_MASK     = 0xFF

      # The bit mask applied to get the account type
      ACCOUNT_TYPE_MASK = 0xF

      def initialize(data = 0)
        @data = data.to_i
      end

      # Returns the internal integer
      def to_i
        @data.to_i
      end

      # Gets the account id
      #
      # @return [Integer] the account id
      def account_id
        get(0, ACCOUNT_ID_MASK)
      end

      # Sets the account_id
      #
      # @return [Integer] the account id
      def account_id=(value)
        set(0, ACCOUNT_ID_MASK, value)
      end

      # Gets the instance
      #
      # @return [Integer] the account id
      def instance
        get(32, INSTANCE_MASK)
      end

      # Sets the instance
      def instance=(value)
        set(32, INSTANCE_MASK, value)
      end

      # Gets the account type
      #
      # @return [Integer] the account type
      def account_type
        get(52, ACCOUNT_TYPE_MASK)
      end

      # Sets the account type
      def account_type=(value)
        set(52, ACCOUNT_TYPE_MASK, value)
      end

      # Gets the universe
      #
      # @return [Integer] the universe
      def universe
        get(56, UNIVERSE_MASK)
      end

      # Sets the universe
      def universe=(value)
        set(56, UNIVERSE_MASK, value)
      end

      private

      # @api private
      def get(offset, mask)
        (@data >> offset) & mask
      end

      # @api private
      def set(offset, mask, value)
        @data = (@data & ~(mask << offset)) | ((value & mask) << offset)
      end
    end
  end
end
